/********************************************************************************
 *   opcode -- operator code retrieval utility for dciBalance.                  *
 *                                                                              * 
 *   Copyright (C) 2004-2007 dci (dev@cpp.in)                                   *
 *                                                                              *
 *   This program is free software; you can redistribute it and/or              *
 *   modify it under the terms of the GNU General Public License version 2      *
 *   as published by the Free Software Foundation.                              *
 *                                                                              *
 *   This program is distributed in the hope that it will be useful,            *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of             *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU          *
 *   General Public License version 2 for more details.                         *
 *                                                                              *
 *   You should have received a copy of the GNU General Public License          *
 *   version 2 along with this library; if not, write to the Free Software      *
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA  *
 ********************************************************************************
 */


#include <windows.h>
#include <ril.h>
//
extern "C"
{
	DWORD  GetEventData    ( HANDLE );
	BOOL   SetEventData    ( HANDLE, DWORD );
};

/* RIL */
void ( * g_pOnConnect ) ( );
void ( * g_pOnDisconnect ) ( );
HMODULE g_hRilDLL = NULL;
HANDLE  g_hRil = NULL;
HRESULT g_hCommand = NULL;
HANDLE  g_hResultEvent = NULL;

void ic_RIL_NotifyCallback ( DWORD dwCode, const void * lpData, DWORD cbData, DWORD dwParam )
{
	/* we are waiting and got ssd */
	if ( g_hCommand && ( dwCode == RIL_NOTIFY_SUPSERVICEDATA ) )
	{
		/* set result */
		void * mem = malloc ( cbData );
		memcpy ( mem, lpData, cbData );
		SetEventData ( g_hResultEvent, ( DWORD ) mem );
		SetEvent ( g_hResultEvent );
		/* we are no longer waiting */
		g_hCommand = NULL;

	}
	/* we are just listening and we got connect/disconnect */
	if ( ( dwCode == RIL_NOTIFY_CONNECT ) && g_pOnConnect )
	{
		g_pOnConnect ( );
	}
	else if ( ( dwCode == RIL_NOTIFY_DISCONNECT ) && g_pOnDisconnect )
	{
		g_pOnDisconnect ( );
	}
}

void ic_RIL_ResultCallback ( DWORD dwCode, HRESULT hrCmdID, const void * lpData, DWORD cbData, DWORD dwParam )
{
	if ( ( ( dwCode & RIL_NCLASS_ALL ) == ( RIL_NCLASS_FUNCRESULT & RIL_NCLASS_ALL ) ) && ( hrCmdID == g_hCommand ) )
	{ /* this is result of our call */
		if ( dwCode == RIL_RESULT_OK )
		{ /* no error */
			if ( cbData )
			{
				/* set result */
				void * mem = malloc ( cbData );
				memcpy ( mem, lpData, cbData );
				SetEventData ( g_hResultEvent, ( DWORD ) mem );
				SetEvent ( g_hResultEvent );
				/* we are no longer waiting */
				g_hCommand = NULL;

			}
			/* wait for notify in some cases */
		}
		else
		{ /* abort */
			SetEventData ( g_hResultEvent, 0 );
			SetEvent ( g_hResultEvent );
		}
	}
}

bool ic_RIL_Init ( unsigned char rilnum, void ( * pOnConnect ) ( ) = NULL, void ( * pOnDisconnect ) ( ) = NULL )
{
	g_pOnConnect = pOnConnect;
	g_pOnDisconnect = pOnDisconnect;
	/* */
	g_hRilDLL = LoadLibrary ( L"ril.dll" );
	if ( g_hRilDLL )
	{
		/* try to init */
		FARPROC call = GetProcAddress ( g_hRilDLL, L"RIL_Initialize" );
		if ( call && SUCCEEDED ( ( ( HRESULT ( * ) ( DWORD, void *, void *, DWORD, DWORD, HANDLE * ) ) call ) ( rilnum, ic_RIL_ResultCallback, ic_RIL_NotifyCallback, 0xFFFFFFFF, 0, &g_hRil ) ) )
		{
			g_hResultEvent = CreateEvent ( NULL, 0, 0, L"ResultEvent" );
			SetEventData ( g_hResultEvent, 0 );
			return 1;
		}
		FreeLibrary ( g_hRilDLL );
		g_hRil = NULL;
		g_hRilDLL = NULL;
	}
	return 0;
}

bool ic_RIL_NCode ( wchar_t * reply, size_t max_reply )
{
	bool ret = 0;
	if ( g_hRil && ( g_hCommand == NULL ) )
	{
		/* previous result pending? */
		wchar_t * oldresult = ( wchar_t * ) GetEventData ( g_hResultEvent );
		if ( oldresult )
		{
			free ( oldresult );
		}
		SetEventData ( g_hResultEvent, 0 );
		/* exec */
		FARPROC call = GetProcAddress ( g_hRilDLL, L"RIL_GetCurrentOperator" );
		g_hCommand = ( ( HRESULT ( * ) ( HANDLE hRil, DWORD dwFormat ) ) call ) ( g_hRil, RIL_OPFORMAT_LONG | RIL_OPFORMAT_SHORT | RIL_OPFORMAT_NUM );
		if ( SUCCEEDED ( g_hCommand ) )
		{
			/* wait for new result */
			if ( WaitForSingleObject ( g_hResultEvent, 1000 ) == WAIT_OBJECT_0 )
			{
				RILOPERATORNAMES * result = ( RILOPERATORNAMES * ) GetEventData ( g_hResultEvent );
				if ( result )
				{
					ret = 1;
					/* */
					for ( size_t i = 0; i < min ( strlen ( result->szNumName ) + 1, max_reply ); i++ ) reply[i] = result->szNumName[i];
					reply[max_reply - 1] = 0;
					/* */
					free ( result );
					SetEventData ( g_hResultEvent, 0 );
				}
			}
		}
		g_hCommand = NULL;
	}
	/* */
	return ret;
}


void ic_RIL_Free ( )
{
	if ( g_hCommand )
	{
		g_hCommand = NULL;
	}
	if ( g_hResultEvent )
	{
		/* result pending? */
		wchar_t * result = ( wchar_t * ) GetEventData ( g_hResultEvent );
		if ( result )
		{
			free ( result );
		}
		CloseHandle ( g_hResultEvent );
		g_hResultEvent = NULL;
	}
	if ( g_hRil && g_hRilDLL )
	{
		FARPROC call = GetProcAddress ( g_hRilDLL, L"RIL_Deinitialize" );
		if ( call ) ( ( HRESULT ( * ) ( HANDLE ) ) call ) ( g_hRil );
		g_hRil = NULL;
	}
	if ( g_hRilDLL )
	{
		FreeLibrary ( g_hRilDLL );
	}
}




void main ( )
{
	if ( ic_RIL_Init ( 1 ) )
	{
		wchar_t opcode[32];
		if ( ic_RIL_NCode ( opcode, 32 ) )
		{
			MessageBox ( NULL, opcode, L"dci", MB_ICONINFORMATION | MB_TOPMOST );
		}
		else
		{
			MessageBox ( NULL, L"Unable to work with network!", L"dci", MB_ICONEXCLAMATION | MB_TOPMOST );
		}
		ic_RIL_Free ( );
	}
	else
	{
		MessageBox ( NULL, L"Unable to init RIL!", L"dci", MB_ICONEXCLAMATION | MB_TOPMOST );
	}
}
